@@ -1,5 +1,12 @@
 from __future__ import absolute_import, unicode_literals
 import sys
+import base64
+
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
+
 from difflib import SequenceMatcher
 from django.db import models
 from django.utils.encoding import force_unicode
@@ -15,9 +22,39 @@ else:
 
 from versioning import _registry
 
+try:
+    str = unicode  # Python 2.* compatible
+    string_types = (basestring,)
+    integer_types = (int, long)
+except NameError:
+    string_types = (str,)
+    integer_types = (int,)
+
+PICKLED_MARKER = 'pickled:'
+STR_MARKER = 'str:'
+
 dmp = diff_match_patch()
 
 
+def encode(val):
+    if not isinstance(val, string_types):
+        return PICKLED_MARKER + base64.standard_b64encode(
+            pickle.dumps(val, protocol=pickle.HIGHEST_PROTOCOL)
+        ).decode('ascii')
+    return STR_MARKER + force_unicode(val)  # prevent to user to falsify PICKLED_MARKER
+
+
+def decode(val):
+    if val.startswith(PICKLED_MARKER):
+        try:
+            return pickle.loads(base64.standard_b64decode(val[len(PICKLED_MARKER):].encode('ascii')))
+        except Exception:
+            pass
+    elif val.startswith(STR_MARKER):
+        return val[len(STR_MARKER):]
+    return val
+
+
 def revisions_for_object(instance):
     from .models import Revision
     return Revision.objects.get_for_object(instance)
@@ -31,13 +68,14 @@ def diff(txt1, txt2):
 
 def set_field_data(obj, field, data):
     field_class = obj._meta.get_field(field)
-    if (field_class.null) and (data == 'None'):
+    # data = decode(data)
+    if field_class.null and data == 'None':
         data = None
     elif isinstance(field_class, models.BooleanField) or isinstance(field_class, models.NullBooleanField):
         if data == 'True':
             data = True
         elif data == 'False':
-            data = False              
+            data = False
     elif isinstance(field_class, models.ForeignKey):
         if data == 'None':
             data = field_class.rel.to()
@@ -60,6 +98,7 @@ def get_field_data(obj, field):
             data = None
     else:
         data = getattr(obj, field)
+    # return encode(data)
     return force_unicode(data)
 
 
